/*
 * Repository.java
 *
 * Created for HW2 of Analysis of Software Artifacts
 * $Id: Repository.java,v 1.2 2004/02/27 02:43:01 ahmedb Exp $
 * $Log: Repository.java,v $
 * Revision 1.2  2004/02/27 02:43:01  ahmedb
 * when you retrieve do not unbind...
 * created a new function to unbind (remove())
 *
 * Revision 1.1  2004/02/26 20:31:42  ahmedb
 * new classes for Replication Management
 *
 * Revision 1.2  2004/02/16 22:04:17  hisashiy
 * Modified/added debug printing. Removed pid handling from Java code. List-mode now handles very first time access to JNDI (it now creates root node).
 *
 */
package edu.cmu.fantango.servlet;

import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NameAlreadyBoundException;
import javax.naming.NameClassPair;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;

/**
 * Repository class is a representation of commands that is managed
 * by the RepMan. The class uses JNDI to register and retrieve command
 * objects.
 *
 * @author hisashiy
 */
public class Repository {

	protected static final String JNDI_INITIAL_CONTEXT_FACTORY;
	protected static final String JNDI_URL_PKG_PREFIXES;
	protected static final String JNDI_PROVIDER_URL;
	static {
		//jndi.properties
		//NOTE: jndi.properties might be searched through JNDI classes
		//     by default. In that case, it searches from the PATH.
		ResourceBundle rb = ResourceBundle.getBundle("jndi", Locale.getDefault());
		JNDI_INITIAL_CONTEXT_FACTORY =rb.getString("initial_context_factory");
		JNDI_URL_PKG_PREFIXES = rb.getString("url_pkg_prefixes");
		JNDI_PROVIDER_URL = rb.getString("provider_url");
	}

	protected Context getInitialContext() throws RepositoryException {
		Context initial = null; //initial context.
		try {
			Properties p = new Properties();
			p.put(Context.INITIAL_CONTEXT_FACTORY, JNDI_INITIAL_CONTEXT_FACTORY);
			p.put(Context.URL_PKG_PREFIXES, JNDI_URL_PKG_PREFIXES);
			p.put(Context.PROVIDER_URL, JNDI_PROVIDER_URL);
			initial = new InitialContext(p);
		} catch (NamingException e) {
			throw new RepositoryException("Repository#getInitialContext(): "
					+ "Failed to get Context: JNDI failure.\n", e);
		}
		return initial;
	}

	protected Context getContext() throws RepositoryException {
		Context root = null; //root node used for RepMan.
		try {
			Context context = getInitialContext();
			Context c = (Context)context.lookup("java:");

			try {
				root = (Context)c.lookup("repman");
			} catch (NamingException e1) {
				//First time to access. Otherwise already fails the above.
				c.createSubcontext("repman");
				root = (Context)c.lookup("repman");
			}
		} catch (NamingException e2) {
				throw new RepositoryException("Repository#getContext(): "
						+ "Failed to get Context: JNDI failure.\n", e2);
		}
		
		if(root == null) {
			throw new RepositoryException("Repository#getContext(): "
					+ "Failed to get Context: context null.\n");
		}
		
		return root;
	}

	/**
	 * Registers the command that is set as the argument. It will fail to
	 * register the command if the label was already registered with other
	 * command object.
	 * 
	 * @param cmd command object to be registered.
	 * @throws RepositoryException thrown when registration fails.
	 */
	public void register(Command cmd) throws RepositoryException {
		Context c = getContext();
		try {
			c.bind(cmd.getLabel(), cmd);
		} catch (NameAlreadyBoundException e1) {
			throw new RepositoryException("Repository#register(): "
					+ "Name [" + cmd.getLabel()
					+ "] already bound to JNDI", e1);
		} catch (NamingException e2) {
			throw new RepositoryException("Repository#register(): "
					+ "Binding new command [" + cmd.getLabel()
					+ "] failed", e2);
		}
	}

	/**
	 * Retrieves command object by label set as the argument.
	 * It will return some command when found or null when command was
	 * not found. Exception is thrown only when there are failures on
	 * implemented repositories (i.e. JNDI).
	 * 
	 * @param label The label to search the command object.
	 * @return Returns the command that corresponds to the label.
	 * Returns null if there was no command registered by the label given.
	 * @throws RepositoryException Thrown when fails to retrieve.
	 */
	public Command retrieve(String label) throws RepositoryException {
		Context c = getContext();
		Command cmd = null;
		try {
			cmd = (Command)c.lookup(label);
			//c.unbind(label); when retrieving do not unbind...
		} catch (NamingException e) {
			throw new RepositoryException("Repository#retrieve(): "
					+ "Lookup/Unbind failed on [" + label + "]", e);
		}
		return cmd;
	}

	public void remove(String label) throws RepositoryException {
		Context c = getContext();
		Command cmd = null;
		try {
			cmd = (Command)c.lookup(label);
			c.unbind(label); //NameNotFoundException should not occur because it is already found.
		} catch (NamingException e) {
			throw new RepositoryException("Repository#retrieve(): "
					+ "Lookup/Unbind failed on [" + label + "]", e);
		}
	}

	/**
	 * Retieves list containing all of commands that are currently held
	 * by the repository.
	 * 
	 * @return Returns a list of commands (of Command class) that is currently
	 * registered to the repository.
	 * @throws RepositoryException Thrown when fails to retrieve.
	 */
	public List retrieveAll() throws RepositoryException {
		//do getContext once even though it is not used.
		//to create the JNDI name tree.
	    getContext();

		Context c = getInitialContext();
		List list = new LinkedList();
		NamingEnumeration enum = null;

		try {
			enum = c.list("java:repman");
			Context context = (Context)c.lookup("java:repman");
			while (enum.hasMore()) {
				NameClassPair pair = (NameClassPair)enum.next();
				Command cmd = (Command)context.lookup(pair.getName());
				list.add(cmd);
			}
		} catch (NamingException e) {
			throw new RepositoryException("Repository#retrieveAll(): "
					+ "List failed", e);
		} finally {
			try {
				if(enum != null) enum.close();
			} catch (NamingException ignore) {
				//ignored intentionally.
			}
		}
		return list;
	}
}
